// Imperative and Object-Oriented Programming Methodology
// Uppsala University - Copyright © 2020 Henrik Alfken(Schulze), Tobias Wrigstad
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "list.h"

typedef struct list list_t;
typedef struct node node_t;

struct list
{
  node_t *dummy;
};

struct node
{
  node_t *prev;
  node_t *next;
  int element;
};

/// Skapar en ny node med ett givet element (du får ändra denna om du vill)
static node_t *node_create(int element)
{
  node_t *result = malloc(sizeof(node_t));
  *result = (node_t) {
    .element = element
  };
  return result;
}
  
list_t *ioopm_list_create()
{
  list_t *result = calloc(1, sizeof(list_t));
  node_t *dummy = node_create(-666666);
  dummy->prev = dummy;
  dummy->next = dummy;
  result->dummy = dummy;
  return result; 
}

void ioopm_list_destroy(list_t *list)
{
  node_t *stop = list->dummy;
  node_t *cursor = stop->next;

  while (cursor != stop)
    {
      node_t *to_delete = cursor;
      cursor = cursor->next;
      free(to_delete);
    }

  free(list->dummy);
  free(list);
}

void ioopm_list_prepend(list_t *list, int element)
{
  node_t *n = node_create(element);
  n->next = list->dummy->next; 
  n->prev = list->dummy;
  n->prev->next = n;
  n->next->prev = n;
}

void ioopm_list_append(list_t *list, int element)
{
  node_t *n = node_create(element);
  n->next = list->dummy; 
  n->prev = list->dummy->prev;
  n->prev->next = n;
  n->next->prev = n;
}

/// FIXME: obs hanterar ej valida index
static bool valid_index(list_t *list, int index)
{
  int size = ioopm_list_size(list);
  return 0 <= index && index < size;
}

bool ioopm_list_remove(list_t *list, int index)
{
  if (index < 0)
    {
      index = ioopm_list_size(list) + index;
    }
  
  if (valid_index(list, index) == false) return false;
  
  node_t *stop = list->dummy;
  node_t *cursor = stop->next;

  int i = 0;
  while (cursor != stop)
    {
      if (i == index)
        {
          cursor->prev->next = cursor->next;
          cursor->next->prev = cursor->prev;
          free(cursor);
          return true;
        }
      cursor = cursor->next;
      ++i;
    }

  return false;
}


int *ioopm_list_get(list_t *list, int index)
{
  if (index < 0)
    {
      index = ioopm_list_size(list) + index;
    }
  
  if (valid_index(list, index) == false) return NULL;
  
  node_t *stop = list->dummy;
  node_t *cursor = stop->next;

  int i = 0;
  while (cursor != stop)
    {
      if (i == index) return &cursor->element;
      cursor = cursor->next;
      ++i;
    }

  return NULL;
}

/// Du får ändra på denna kod om du vill -- t.ex. för att
/// den inte funkar enligt din design, eller för att det
/// är enklare att skriva om den än att förstå den!
bool ioopm_list_contains(list_t *list, int element)
{
  node_t *stop = list->dummy;
  node_t *cursor = stop->next;

  while (cursor != stop)
    {
      if (cursor->element == element) return true;
      cursor = cursor->next;
    }

  return false;
}

int ioopm_list_size(list_t *list)
{
  node_t *stop = list->dummy;
  node_t *cursor = stop->next;

  int size = 0;
  while (cursor != stop)
    {
      cursor = cursor->next;
      ++size;
    }

  return size;
}
