// Imperative and Object-Oriented Programming Methodology
// Uppsala University - Copyright © 2020 Henrik Alfken(Schulze), Tobias Wrigstad
public class Set<T extends Comparable<T>> {
  private int size = 0;
  private int maxCapacity = -1;
  private final T[] elements;

  public Set(int i) {
    maxCapacity = i;
    elements = (T[]) new Comparable[maxCapacity];
  }

  public void add(T elem) throws SetFullException {
    if (size + 1 > maxCapacity) {
      throw new SetFullException(" - The set is full. ");
    } else if (size == 0) {
      elements[size] = elem; // The first element inserted.
      ++size;                // The current size is now = 1.
    } else {    // currentSize > 0 :
      int i = 0;
      while (i < size && elem.compareTo(elements[i]) >= 0) {
        ++i;
      }                           // The element 'elem' should be inserted at index i,
      for (int j = size; j > i ; j--) {  // but first move all existing elems ...
        elements[j] = elements[j-1];  // ... to make room for an element at index i.
      }
      if (i == 0 || elem.compareTo(elements[i-1]) > 0) {  // Insert 'elem' at index i:
        elements[i] = elem;
        ++size;
      }
// If elem.compareTo(elements[i-1]) = 0, 'elem' is already in the set - don't insert it again.
    }
  }

  public boolean contains(T elem) {
    boolean result = false;
    for (int i = 0; i < size; i++) {
      if (elem.compareTo(elements[i]) == 0) { result = true; }
    }
    return result;
  }

  public int size() {
    return size;
  }

  public boolean equals(Set other) {
    boolean result = false;
    if (size == ((Set) other).size()) {
      result = true;
      for (int i = 0; result && i < size; ++i) {
        result = (this.elements[i].equals(((Set) other).elements[i]));
      }
    }
    return result;
  }

  @Override
  public boolean equals(Object other) {
    if (other instanceof Set) { return this.equals((Set)other); }
    else { return super.equals(other); }
  }
}
